@@ -24,10 +24,12 @@
 #include <aconf.h>
 
 #include <stdio.h>
+#include <unistd.h>
 #include "Gfx.h"
 #include "GfxState.h"
 #include "GfxFont.h"
 #include "GPOutputDev.h"
+#include "FontFile.h"
 
 #ifdef HAVE_FONT_EMBEDDING
 #  include "gpdf-g-switch.h"
@@ -80,9 +82,9 @@ GPOFontMap::~GPOFontMap()
   }
 }
 
-void GPOFontMap::startDoc(XRef *xref)
+void GPOFontMap::startDoc(XRef *xrefA)
 {
-  this->xref = xref;
+  this->xref = xrefA;
   
   if (embeddedFonts != NULL)
     g_hash_table_destroy(embeddedFonts);
@@ -202,7 +204,21 @@ GnomeFontFace *GPOFontMap::getFontFaceEmbedded(GfxFont *font)
     return gff;
   }
 
+#ifdef PDF_DEBUG
   switch (font->getType()) {
+  case fontType1: g_message("fontType1"); break;
+  case fontType1C: g_message("fontType1C"); break;
+  case fontTrueType: g_message("fontTrueType"); break;
+  case fontCIDType2: g_message("fontCIDType2"); break;
+  case fontCIDType0: g_message("fontCIDType0"); break;
+  case fontCIDType0C: g_message("fontCIDType0C"); break;
+  case fontType3: g_message("fontType3"); break;
+  case fontUnknownType: g_message("fontUnknownType"); break;
+  }
+#endif /* PDF_DEBUG */
+
+  switch (font->getType()) {
+  case fontTrueType:
   case fontType1:
   case fontType1C: {
     Ref embRef;
@@ -222,6 +238,28 @@ GnomeFontFace *GPOFontMap::getFontFaceEmbedded(GfxFont *font)
       g_assert_not_reached(); /* FIXME */
     strObj.free();
 
+    if (font->getType() == fontTrueType ||
+	font->getType() == fontCIDType2) {
+      /* FIXME THIS NEEDS ERROR CHECKING */
+      gint fd;
+      gchar *temp_name;
+
+      fd = g_file_open_tmp ("gpdf-ttf-XXXXXX", &temp_name, NULL);
+      FILE *f = fdopen (fd, "wb");
+
+      TrueTypeFontFile *ff = new TrueTypeFontFile((char *)contents, length);
+      ff->writeTTF(f);
+      delete ff;
+      g_free(contents);
+
+      fclose(f);
+
+      g_file_get_contents(temp_name, (gchar **)&contents, &length, NULL);
+
+      unlink(temp_name);
+      g_free(temp_name);
+    }
+
     DBG (g_message("Adding font face for %s", font_name));
     
     gff = gpdf_font_face_download ((const guchar *)font_name,
@@ -630,7 +668,13 @@ void GPOutputDev::drawString(GfxState *state, GString *s) /* FIXME */
     return;
 
   gnome_print_gsave(gpc);
-  gnome_print_concat(gpc, state->getTextMat());
+
+  gdouble fontmat[6];
+  memcpy(fontmat, state->getTextMat(), 6 * sizeof(gdouble));
+  fontmat[0] *= state->getHorizScaling();
+  fontmat[2] *= state->getHorizScaling();
+
+  gnome_print_concat(gpc, fontmat);
   gnome_print_moveto(gpc, text_pos_x, text_pos_y);
 
   GnomeGlyphList *ggl = gnome_glyphlist_new();
@@ -658,7 +702,6 @@ void GPOutputDev::drawString(GfxState *state, GString *s) /* FIXME */
     dx = dx * state->getFontSize() + state->getCharSpace();
     if (n == 1 && *p == ' ')
       dx += state->getWordSpace();
-    dx *= state->getHorizScaling();
     dy *= state->getFontSize ();
 
     DBG (if (uLen > 1)
@@ -705,7 +748,7 @@ void GPOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
     *p++ = (guchar)(maskColor.r * 255.999);
     *p++ = (guchar)(maskColor.g * 255.999);
     *p++ = (guchar)(maskColor.b * 255.999);
-    *p++ = alpha ? 0 : 255;
+    *p++ = alpha ? 255 : 0;
   }
 
   gnome_print_rgbaimage(gpc, pixbuf, width, height, width * 4);
